#!/usr/bin/env python
# -*- coding: utf-8 -*-
# vim:ts=4:sw=4:softtabstop=4:smarttab:expandtab
import sys, os, re, shutil, platform, tempfile, imghdr, locale
from PIL import ImageTk, Image
from os.path import expanduser

if sys.platform.startswith('win'):
    os_encoding = locale.getpreferredencoding()
else:
    os_encoding = 'utf-8'

#------------------------------------
# display ImageMagick file selection dialog
#------------------------------------
def GetFileName():
    ''' displays the ImageMagick file selection dialog '''
    # requires Sigil 0.9.8 or higher or PyQt5
    home = expanduser('~')
    from PyQt5.QtWidgets import QApplication, QWidget, QFileDialog
    app = QApplication(sys.argv)
    w = QWidget()
    file_path, filter = QFileDialog.getOpenFileName(w,'Select ImageMagick exe-file', home, 'ImageMagick-Executable (magick.exe)')
   
    return file_path


# wrapper for ImageMagick
def im_wrapper(*args):
    ''' a wrapper for ImageMagick '''
    import subprocess
    startupinfo = None

    # stop the windows console popping up every time the prog is run
    if sys.platform.startswith('win'):
        startupinfo = subprocess.STARTUPINFO()
        startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
        startupinfo.wShowWindow = subprocess.SW_HIDE

    process = subprocess.Popen(list(args), stdout=subprocess.PIPE, stderr=subprocess.PIPE, startupinfo=startupinfo)
    stdout, stderr = process.communicate()

    returncode = process.returncode
    return stdout.decode('utf-8', 'ignore'), stderr.decode('utf-8', 'ignore'), returncode

# main routine
def run(bk):
    ''' the main routine '''

    #---------------------------------
    # get preferences
    #---------------------------------
    prefs = bk.getPrefs()
    if prefs == {}:
      prefs['maxWidth'] = '600';
      prefs['debug'] = False;
      bk.savePrefs(prefs)

    # Get ImageMagick path
    magick_exe_path = None
    magick_exe_path = prefs.get('magick_exe_path', None)
    if not magick_exe_path or not os.path.isfile(magick_exe_path):
        magick_exe_path = GetFileName()
        if magick_exe_path:
            prefs['magick_exe_path'] = magick_exe_path
            bk.savePrefs(prefs)


    # get pref values
    magick_exe_path = prefs.get('magick_exe_path', None)
    debug = prefs.get('debug', False) # show debug messages
    maxWidth = prefs.get('maxWidth', '600')

    # make sure that the ImageMagick binary was found
    if not magick_exe_path or not os.path.isfile(magick_exe_path): 
        print('ImageMagick binary not found!\nPlease check path in pref.\nClick OK to close the Plugin Runner window.')
        return -1

    # get ebook root folder
    ebook_root = bk._w.ebook_root

    # get os temp folder
    temp_dir = tempfile.gettempdir()

    # get img_files
    img_manifest_items = []
    for manifest_id, href, mime in bk.manifest_iter():
        if mime == "image/jpeg" or mime == "image/png":
            img_manifest_items.append((manifest_id, href, mime))
    
    if img_manifest_items == []:
        print('No img files found.\nClick OK to close the Plugin Runner window.')
        return 0

    # process all manifested jpeg/png files in the Images folder
    bytes_saved = 0
    for manifest_id, href, mime in img_manifest_items:
        img_file_name = os.path.basename(href)
        processed_file_path = os.path.join(temp_dir, img_file_name)

        # Sigil 1.x and higher supports custom image paths
        if  bk.launcher_version() >= 20190927:
            original_img_path = os.path.abspath(os.path.join(ebook_root, bk.id_to_bookpath(manifest_id)))
        else:
            original_img_path = os.path.abspath(os.path.join(ebook_root, 'OEBPS', href))

        # double-check the file header to make sure it's really a img file
        # using pillow instead of imghdr 
        im = Image.open(original_img_path)
        (w,h) = im.size
        imgFormat = im.format
        
        if ((imgFormat != 'JPEG') and (imgFormat != 'PNG')) or w < int(maxWidth) :
            if debug: print('Ignoring: ', img_file_name, 'is a {} file; Width= {}\n'.format(imgFormat, w))
            continue
        
        # assemble args
        args = [magick_exe_path, 'mogrify','-resize', maxWidth, '-path', temp_dir, original_img_path]

        # run ImageMagick
        print('\nresizing: {} ...'.format(img_file_name))
        if debug: print(args)
        stdout, stderr, returncode = im_wrapper(*args)

        # display error messages, if ImageMagick failed
        if returncode != 0:
            print('ImageMagick failed!\n{}\nClick OK to close the Plugin Runner window'.format(stdout + stderr, returncode))
            return -1

        # check whether ImageMagick created an optimized file
        if os.path.isfile(processed_file_path):
            with open(processed_file_path, 'rb') as file:
                 optimized_image_data = file.read()
            bk.writefile(manifest_id, optimized_image_data)
            print('Image replaced with resized image.')

            # delete temp file
            os.remove(processed_file_path)
        else:
            # if there's no output file, optimized and original files sizes are identical
            print(stderr)
            print('Skipped')

    # display endmessage
    print('\nDone.\nClick OK to close the Plugin Runner window.')

    return 0

def main():
    print('I reached main when I should not have\n')
    return -1

if __name__ == "__main__":
    sys.exit(main())
